#pragma once
#ifndef __MutuallyAttachable_H__
#define __MutuallyAttachable_H__

// Usage: Suppose you want to have two classes A and B that are mutually connected to each other.
//
//        Derive the class A from MutuallyAttachable<A, B> where A is the class you're deriving
//        and B is the other class you want to make mutual attachments with.
//        Derive the class B from MutuallyAttachable<B, A>.
//
//        Call MutuallyAttachable<A, B>::CreateConnection(A, B) to create a connection.
//        Call MutuallyAttachable<A, B>::DestroyConnection(A, B) to destroy a connection.
//
//        Call A::GetAttached() to get a const reference to std::set<B *>.
//        Call B::GetAttached() to get a const reference to std::set<A *>.
//
//        If A or B is destroyed while it has non-zero connections, it will automatically remove the connections.

template <typename T, typename U> class MutuallyAttachable
{
public:
	auto GetAttached() -> const std::set<U *> &;

	static void CreateConnection(MutuallyAttachable<T, U> & MutuallyAttachable0, MutuallyAttachable<U, T> & MutuallyAttachable1);
	static void DestroyConnection(MutuallyAttachable<T, U> & MutuallyAttachable0, MutuallyAttachable<U, T> & MutuallyAttachable1);

protected:
	MutuallyAttachable();
	virtual ~MutuallyAttachable();

private:
	MutuallyAttachable(const MutuallyAttachable<T, U> &) = delete;
	MutuallyAttachable<T, U> & operator =(const MutuallyAttachable<T, U> &) = delete;

	void AttachWithoutReciprocating(MutuallyAttachable<U, T> & MutuallyAttachable);
	void DetachWithoutReciprocating(MutuallyAttachable<U, T> & MutuallyAttachable);

	std::set<U *>		m_Attached;

	friend class MutuallyAttachable<U, T>;

//public:
	//auto GetAttached() -> decltype((m_Attached));
	//auto GetAttached() -> const std::set<MutuallyAttachable<U, T> *> &;
};

#include "MutuallyAttachable.hpp"

#endif // __MutuallyAttachable_H__
